#include <iostream>
#include <math.h>
#include <vector>
#include <FL/gl.h>
#include <FL/glut.H>

using namespace std;

#include "flipper.h"

Flipper::Flipper()
    : Poly (), angledelta (0.0), angle (0.0)
{
    /*
    edges.push_back (Line (Point_3d ( 0, 25, 0), Point_3d ( 0,-25, 0)));
    edges.push_back (Line (Point_3d ( 0,-25, 0), Point_3d ( 100,-20, 0)));
    edges.push_back (Line (Point_3d ( 100,-20, 0), Point_3d ( 100, 20, 0)));
    edges.push_back (Line (Point_3d ( 100, 20, 0), Point_3d ( 0, 25, 0)));
    */
}

Flipper::Flipper (const Point_3d &pos, 
                  const std::vector<Line> &vert, 
                  const bool &isleft,
                  const Vector_3d &vec)
    : Poly (pos, vert), angledelta (0.0), angle (0.0)
{
    if (isleft)
        orientation = 1.0;
    else
        orientation =-1.0;
}

Flipper::Flipper (const Flipper &that)
{
    *this = that;
}

Flipper &Flipper::operator= (const Flipper &that)
{
    if (this == &that)
        return *this;
    this->position = that.position;
    this->velocity = that.velocity;
    this->collided = that.collided;
    this->edges = that.edges;
    this->angledelta = that.angledelta;
    this->angle = that.angle;
    this->orientation = that.orientation;
    return *this;
}

bool Flipper::collide (Object &other)
{
    bool retval = false;
    vector<Line>::iterator start = edges.begin();
    vector<Line>::iterator end = edges.end();
    for (; start != end; ++start)
    {
        Line angled = rotate_points (*start);

        Point_3d pta = angled.get_a();
        pta.set_x (position.get_x() + pta.get_x());
        pta.set_y (position.get_y() + pta.get_y());

        Point_3d ptb = angled.get_b();
        ptb.set_x (position.get_x() + ptb.get_x());
        ptb.set_y (position.get_y() + ptb.get_y());

        Line offset = Line (pta, ptb);
        retval = retval || offset.collide (other);//offset.collide (other);
        *start = angled;
    }
    other.set_col (retval);
    return retval;
}

void Flipper::resolve_collision (Object &other)
{
}

Line Flipper::rotate_points (const Line &old)
{
    Point_3d origa = old.get_a();
    double origax = origa.get_x();
    double origay = origa.get_y();

    Point_3d origb = old.get_b();
    double origbx = origb.get_x();
    double origby = origb.get_y();

    double radangledelta = angledelta * (M_PI / 180);
    //x' = x cos f - y sin f
    //y' = y cos f + x sin f
    double newax = origax * cos (orientation * radangledelta) - 
                   origay * sin (orientation * radangledelta);
    //cout << origax << " to " << newax << endl;
    double neway = origay * cos (orientation * radangledelta) +
                   origax * sin (orientation * radangledelta);
    //cout << origay << " to " << neway << endl;
        
    double newbx = origbx * cos (orientation * radangledelta) - 
                   origby * sin (orientation * radangledelta);
    //cout << origbx << " to " << newbx << endl;
    double newby = origby * cos (orientation * radangledelta) +
                   origbx * sin (orientation * radangledelta);
    //cout << origby << " to " << newby << endl;
    Point_3d newa = Point_3d (newax, neway, 0);
    Point_3d newb = Point_3d (newbx, newby, 0);

    return Line (newa, newb);
}

void Flipper::update()
{
    //cout << angle << endl;
    /*if (angle == 0.0)
        angledelta =  1.0;
    else*/ 
    if (angle >= 50 && angledelta > 0.0)
        angledelta = 0.0;
    else if (angle <= 0.0  && angledelta < 0.0)
        angledelta = 0.0;
    angle += angledelta;
}

void Flipper::draw()
{
    vector<Line>::iterator start = edges.begin();
    vector<Line>::iterator end = edges.end();
    glTranslated (position.get_x(), position.get_y(), 0.0);
    for (; start != end; ++start)
    {
        glPushMatrix();
            //glRotated (orientation * angle, 0, 0, 1.0);
            start->draw();
        glPopMatrix();
    }
}

double Flipper::get_angle() const
{
    return angle;
}

void Flipper::set_angle (const double &newang)
{
    angledelta = newang;
}
