#include <iostream>
#include <FL/gl.H>
#include <FL/glut.H>

using namespace std;

#include "my_gl_window.h"

My_Gl_Window::My_Gl_Window (const int &X, const int &Y, 
                            const int &W, const int &H, 
                            const char *L) 
    : Fl_Gl_Window ( X, Y, W, H, L), ypos ( 0.0), 
      theball (Ball (Point_3d (0 * (-w() + 350), 0 * (h() - 200), 0), 20.0)),
      playfield (vector<Poly> ())
{
    init_play_field();
    lflip = Flipper (Point_3d (-w() + 320,-h() + 160, 0), 
                     init_flipper (), 
                     true);
    rflip = Flipper (Point_3d ( w() - 320,-h() + 160, 0), 
                     init_flipper (false), 
                     false);
    ball_reset();
    Fl::add_timeout (FPS, My_Gl_Window::timer_cb, (void*)this);
    end();
}

void My_Gl_Window::timer_cb (void *userdata)
{
    My_Gl_Window *mygl = (My_Gl_Window*) userdata;
    mygl->update();
    mygl->redraw();
    Fl::repeat_timeout (FPS, My_Gl_Window::timer_cb, userdata);
}

void My_Gl_Window::fix_viewport (const int &W, const int &H) 
{
    ///*
    //GLfloat ratio = W / H;

    glLoadIdentity ();
    glViewport ( 0, 0, W, H);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    glOrtho (-W, W,-H, H,-20.0, 20.0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    //*/

    /*
    GLfloat ratio = W / H;
    glViewport(0, 0, (GLsizei)W, (GLsizei)H);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-1.5*ratio, 1.5*ratio, -1.5, 1.5, -10.0, 10.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    */
}

void My_Gl_Window::resize (const int &X, const int &Y, 
                           const int &W, const int &H)
{
    Fl_Gl_Window::resize ( X, Y, W, H);
    fix_viewport ( W, H);
    redraw();
}

bool My_Gl_Window::in_plunge_spot ()
{
    double ballxpos = theball.get_pos().get_x();
    double ballypos = theball.get_pos().get_y();
    bool inarea = false;
    if (ballxpos > w() - 130 && ballxpos < w() - 50)
        if (ballypos >-h() + 50 && ballypos < 0)
            inarea = true;
    return inarea;
}

int My_Gl_Window::key_input (const int &eventkey)
{
    switch (eventkey)
    {
    case 'z':
        lflip.set_angle ( 10.0);
        break;
    case '/':
        rflip.set_angle ( 10.0);
        break;
    case ' ':
        if (in_plunge_spot())
            theball.set_vel (Vector_3d ( 0, 28, 0));
        break;
    default:
        if (lflip.get_angle () > 0)
        {
            lflip.set_angle (-10.0);
            cout << lflip.get_angle () << endl;
        }
        if (rflip.get_angle () > 0)
        {
            rflip.set_angle (-10.0);
            cout << rflip.get_angle () << endl;
        }
        return 0;
    }
    redraw();
    return 1;
}

int My_Gl_Window::handle (int hand)
{
    int exit = Fl_Gl_Window::handle (hand);
    switch (hand)
    {
    case FL_FOCUS:
    case FL_UNFOCUS:
        return 1;
    case FL_KEYBOARD:
        return key_input (Fl::event_key());
    default:
        if (lflip.get_angle () > 0)
        {
            lflip.set_angle (-10.0);
            cout << lflip.get_angle () << endl;
        }
        if (rflip.get_angle () > 0)
        {
            rflip.set_angle (-10.0);
            cout << rflip.get_angle () << endl;
        }
        return exit;
    }
    return exit;
}

/*
Poly My_Gl_Window::init_inner_wall ()
{
    return Poly();
}
*/

vector<Line> My_Gl_Window::init_flipper (const bool &isleft)
{
    vector<Line> shape = vector<Line> ();
    double LorR = 1.0;
    if (!isleft )
        LorR = -1.0;
    shape.push_back (Line (Point_3d (LorR *   0, 25, 0), Point_3d (LorR *   0,-25, 0)));
    shape.push_back (Line (Point_3d (LorR *   0,-25, 0), Point_3d (LorR * 100,-20, 0)));
    shape.push_back (Line (Point_3d (LorR * 100,-20, 0), Point_3d (LorR * 100, 20, 0)));
    shape.push_back (Line (Point_3d (LorR * 100, 20, 0), Point_3d (LorR *   0, 25, 0)));
    return shape;
}

Poly My_Gl_Window::init_wall (const double &x1, 
                              const double &y1, 
                              const double &x2, 
                              const double &y2)
{
    vector<Line> wall = vector<Line> ();
    Point_3d a = Point_3d (x1, y1, 0);
    Point_3d b = Point_3d (x2, y2, 0);
    wall.push_back (Line (a, b));
    return Poly (a, wall);
}

void My_Gl_Window::init_play_field()
{
    //left side
    playfield.push_back ( init_wall (-w() + 50, 
                                      h() - 50, 
                                     -w() + 50, 
                                     -h() + 50));
    // bottom l
    playfield.push_back ( init_wall (-w() + 50, 
                                     -h() + 50, 
                                     -75, 
                                     -h() + 50));

    // bottom 2
    playfield.push_back ( init_wall ( 75, 
                                     -h() + 50, 
                                      w() - 50, 
                                     -h() + 50));
     //right side
    playfield.push_back ( init_wall ( w() - 50, 
                                     -h() + 50, 
                                      w() - 50, 
                                      h() - 50));
    // top
    playfield.push_back ( init_wall (-w() + 50, 
                                      h() - 50, 
                                      w() - 50, 
                                      h() - 50));
    //top l arch
    playfield.push_back ( init_wall (-w() + 200,
                                      h() - 50,
                                     -w() + 50,
                                      h() - 200));
    //top r arch
    playfield.push_back ( init_wall ( w() - 200,
                                      h() - 50,
                                      w() - 50,
                                      h() - 200));
    //bot l arch
    playfield.push_back ( init_wall (-w() + 50, 
                                     -h() + 200, 
                                     -w() + 360, 
                                     -h() + 50));
    //bot r arch
    playfield.push_back ( init_wall ( w() - 360, 
                                     -h() + 50, 
                                      w() - 135, 
                                     -h() + 200));
    //r side wall
    playfield.push_back ( init_wall ( w() - 130,
                                     -h() + 50,
                                      w() - 130,
                                      0));//h() - 350));

    // l side wall
    playfield.push_back ( init_wall (-w() + 130,
                                      0,//h() - 350,
                                     -w() + 130,
                                     -h() + 275));

    // l gutter
    playfield.push_back ( init_wall (-w() + 130, 
                                     -h() + 275, 
                                     -w() + 320, 
                                     -h() + 160));
    // r gutter
    playfield.push_back ( init_wall ( w() - 130, 
                                     -h() + 275, 
                                      w() - 320, 
                                     -h() + 160));
}

void My_Gl_Window::ball_reset ()
{
    theball.set_pos (Point_3d ( w() - 100,-h() + 100, 0));
    theball.set_vel (Vector_3d ( 0, 0, 0));
}

void My_Gl_Window::update()
{
    bool ballcol = false;
    vector<Poly>::iterator start = playfield.begin();
    vector<Poly>::iterator end = playfield.end();
    for (; start != end; ++start)
    {
        //testpoly.draw();
        //testline.draw();
        ballcol = ballcol || start->collide (theball);
    }
    //testline.collide (theball);
    //testpoly.collide (theball);
    lflip.update();
    rflip.update();
    ballcol = ballcol || lflip.collide (theball);
    ballcol = ballcol || rflip.collide (theball);
    theball.set_col (ballcol);
    theball.update();

    //testline.update();
    //testpoly.update();
}

void My_Gl_Window::draw() 
{
    if (!valid())
    {
        valid(1);
        fix_viewport(w(), h());
        /*
        // (REFERENCE: SGI 'light.c' EXAMPLE)
        GLfloat mat_ambient[]    = { 1.0, 1.0, 1.0, 1.0 };  // RGBA
        GLfloat mat_diffuse[]    = { 1.0, 1.0, 1.0, 1.0 };  // RGBA
        GLfloat mat_specular[]   = { 1.0, 1.0, 1.0, 1.0 };  // RGBA
        GLfloat light_position[] = { 5.0, 5.0, 0.0, 0.0 };  // XYZ
        glClearColor(0.0, 0.0, 0.0, 0.0);                   // bg color
        glShadeModel(GL_SMOOTH);
        //
        glMaterialfv(GL_FRONT, GL_AMBIENT,   mat_ambient);
        glMaterialfv(GL_FRONT, GL_DIFFUSE,   mat_diffuse);
        glMaterialfv(GL_FRONT, GL_SPECULAR,  mat_specular);
        glMaterialf(GL_FRONT,  GL_SHININESS, 20.0);
        glLightfv(GL_LIGHT0, GL_POSITION, light_position);
        //
        glEnable(GL_LIGHTING);
        glEnable(GL_LIGHT0);
        glEnable(GL_DEPTH_TEST);
        */
    }
    
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glPushMatrix();
        theball.draw();
    glPopMatrix();

    vector<Poly>::iterator start = playfield.begin ();
    vector<Poly>::iterator end = playfield.end ();
    for (; start != end; ++start)
    {
        glColor3f (1.0f, 0.0f, 1.0f);
        glPushMatrix ();
            start->draw ();
        glPopMatrix ();
    }
    glPushMatrix ();
        glColor3d ( 0.5, 0.5, 0.5);
        lflip.draw ();
    glPopMatrix ();
    glPushMatrix ();
        glColor3d ( 0.5, 0.5, 0.5);
        rflip.draw ();
    glPopMatrix();
}
