#ifndef __MY_GL_WINDOW_H__
#define __MY_GL_WINDOW_H__

#define _USE_MATH_DEFINES
#include <math.h>
//
// OpenGL App With FLTK Widgets
// erco 11/08/06
//
#include <FL/Fl.H>
#include <FL/Fl_Gl_Window.H>

#include "objects.h"
#include "ball.h"
#include "line.h"
#include "polygon.h"
#include "flipper.h"

#define FPS (1.0/24.0)
#define gravity (2.0)

// OPENGL WINDOW CLASS
class My_Gl_Window : public Fl_Gl_Window 
{
private:
    // DRAW METHOD
    double ypos;
    Ball theball;
    //Line testline;
    vector<Poly> playfield;
    Flipper lflip, rflip;

    //shape initialization
    //Poly init_inner_wall ();
    Poly init_wall (const double &x1, 
                    const double &x2, 
                    const double &y1, 
                    const double &y2);
    vector<Line> init_flipper (const bool &isleft = true);
    void init_play_field ();

    bool in_plunge_spot();

    void draw();
    void update();

    // FIX OPENGL VIEWPORT
    //     Do this on init or when window's size is changed
    void fix_viewport (const int &W, const int &H);
    // HANDLE WINDOW RESIZING
    void resize (const int &X, const int &Y, const int &W, const int &H);

    int key_input (const int &eventkey);
public:
    // OPENGL WINDOW CONSTRUCTOR
    My_Gl_Window (const int &X, const int &Y, const int &W, const int &H, const char*L=0);

    // HANDLE METHOD
    int handle (int);

    static void timer_cb (void *userdata);

    void ball_reset ();
};

#endif
