#include <cstdlib>
#include <iostream>
#include <cmath>

using namespace std;

#include "point_3d.h"

Point_3d::Point_3d ()
    : xcoord (0), ycoord (0), zcoord (0)
{
}

Point_3d::Point_3d (const double &x, const double &y, const double &z)
    : xcoord (x), ycoord (y), zcoord (z)
{
}

Point_3d::Point_3d (const Point_3d &that)
{
    *this = that;
}

Point_3d &Point_3d::operator= (const Point_3d &that)
{
    if (this == &that) return *this;
    xcoord = that.xcoord;
    ycoord = that.ycoord;
    zcoord = that.zcoord;
    return *this;
}

Vector_3d Point_3d::distance (const Point_3d &that) const
{
    double xcomp = abs (that.xcoord - xcoord);
    double ycomp = abs (that.ycoord - ycoord);
    double zcomp = abs (that.zcoord - zcoord);
    return Vector_3d (xcomp, ycomp, zcomp);
}

double Point_3d::get_x() const
{
    return xcoord;
}

double Point_3d::get_y() const
{
    return ycoord;
}

double Point_3d::get_z() const
{
    return zcoord;
}

void Point_3d::set_x (const double &newx)
{
    xcoord = newx;
}

void Point_3d::set_y (const double &newy)
{
    ycoord = newy;
}

void Point_3d::set_z (const double &newz)
{
    zcoord = newz;
}

Point_3d operator+ (const Point_3d &pt, const Vector_3d &vect)
{
    return Point_3d (pt.get_x() + vect.get_x(),
                     pt.get_y() + vect.get_y(),
                     pt.get_z() + vect.get_z());
}

Vector_3d operator- (const Point_3d &pt1, const Point_3d &pt2)
{
    return Vector_3d (pt1.get_x() - pt2.get_x(),
                      pt1.get_y() - pt2.get_y(),
                      pt1.get_z() - pt2.get_z());
}

Point_3d operator- (const Point_3d &pt, const Vector_3d &vect)
{
    return Point_3d (pt.get_x() - vect.get_x(),
                     pt.get_y() - vect.get_y(),
                     pt.get_z() - vect.get_z());
}

bool operator== (const Point_3d &pt1, const Point_3d &pt2)
{
    return pt1.xcoord == pt2.xcoord &&
           pt1.ycoord == pt2.ycoord &&
           pt1.zcoord == pt2.zcoord;
}

bool operator!= (const Point_3d &pt1, const Point_3d &pt2)
{
    return !(pt1 == pt2);
}

ostream &operator<< (ostream &out, const Point_3d &that)
{
    out << "(" << that.xcoord << endl
        << " " << that.ycoord << endl
        << " " << that.zcoord << ")";
    return out;
}