#include <cstdlib>
#include <cmath>
#include <iostream>

using namespace std;

#include "vector_3d.h"

Vector_3d::Vector_3d ()
    : xcomp(0), ycomp(0), zcomp(0)
{
}

Vector_3d::Vector_3d (const double &x, 
                      const double &y, 
                      const double &z)
    : xcomp(x), ycomp(y), zcomp(z)
{
}

Vector_3d::Vector_3d (const Vector_3d &that)
{
    *this = that;
}

Vector_3d &Vector_3d::operator= (const Vector_3d &that)
{
    if (this == &that) return *this;
    xcomp = that.xcomp;
    ycomp = that.ycomp;
    zcomp = that.zcomp;
    return *this;
}

double Vector_3d::magnitude () const
{
    return sqrt ( pow (xcomp, 2) + pow (ycomp, 2) + pow (zcomp, 2));
}

double Vector_3d::dot_product (const Vector_3d &that) const
{
    return xcomp * that.get_x() + ycomp * that.get_y() + zcomp * that.get_z();
}

Vector_3d Vector_3d::unit_vector () const
{
    double mag = magnitude();
    return (1 / mag) * *this; 
}

double Vector_3d::get_x () const
{
    return xcomp;
}

double Vector_3d::get_y () const
{
    return ycomp;
}

double Vector_3d::get_z () const
{
    return zcomp;
}

void Vector_3d::set_x (const double &x)
{
    xcomp = x;
}

void Vector_3d::set_y (const double &y)
{
    ycomp = y;
}

void Vector_3d::set_z (const double &z)
{
    zcomp = z;
}

Vector_3d operator+ (const Vector_3d &vect1, const Vector_3d &vect2)
{
    return Vector_3d (vect1.get_x() + vect2.get_x(),
                      vect1.get_y() + vect2.get_y(),
                      vect1.get_z() + vect2.get_z());
}

Vector_3d operator- (const Vector_3d &vect1, const Vector_3d &vect2)
{
    return Vector_3d (vect1.get_x() - vect2.get_x(),
                      vect1.get_y() - vect2.get_y(),
                      vect1.get_z() - vect2.get_z());
}

Vector_3d operator* (const Vector_3d &vect, const double &scalar)
{
    return Vector_3d (vect.get_x() * scalar,
                      vect.get_y() * scalar,
                      vect.get_z() * scalar);
}

Vector_3d operator* (const double &scalar, const Vector_3d &vect)
{
    return vect * scalar;
}

Vector_3d operator- (const Vector_3d &that)
{
	return -1.0 * that;
}

ostream &operator<< (ostream &out, const Vector_3d &that)
{
    out << "[" << that.xcomp << endl
        << " " << that.ycomp << endl
        << " " << that.zcomp << "]";
    return out;
}